@@ -64,6 +64,9 @@ void StripShellComment(string* cmd) {
             in++;
           break;
         }
+#if defined(__has_cpp_attribute) && __has_cpp_attribute(clang::fallthrough)
+        [[clang::fallthrough]];
+#endif
 
       case '\'':
       case '"':
@@ -598,11 +601,12 @@ void CallFunc(const vector<Value*>& args, Evaluator* ev, string* s) {
 
   ev->CheckStack();
   const string&& func_name_buf = args[0]->Eval(ev);
-  const StringPiece func_name = TrimSpace(func_name_buf);
-  Var* func = ev->LookupVar(Intern(func_name));
+  Symbol func_sym = Intern(TrimSpace(func_name_buf));
+  Var* func = ev->LookupVar(func_sym);
+  func->Used(ev, func_sym);
   if (!func->IsDefined()) {
     KATI_WARN_LOC(ev->loc(), "*warning*: undefined user function: %s",
-                  func_name.as_string().c_str());
+                  func_sym.c_str());
   }
   vector<unique_ptr<SimpleVar>> av;
   for (size_t i = 1; i < args.size(); i++) {
@@ -613,7 +617,7 @@ void CallFunc(const vector<Value*>& args, Evaluator* ev, string* s) {
   vector<unique_ptr<ScopedGlobalVar>> sv;
   for (size_t i = 1;; i++) {
     string s;
-    Symbol tmpvar_name_sym(Symbol::IsUninitialized{});
+    Symbol tmpvar_name_sym;
     if (i < sizeof(tmpvar_names) / sizeof(tmpvar_names[0])) {
       tmpvar_name_sym = tmpvar_names[i];
     } else {
